from PyQt5 import QtCore, QtWidgets, QtGui
from PyQt5.QtCore import Qt, QMimeData, QRegExp
from PyQt5.QtWidgets import QTreeWidgetItem, QFrame, QFileDialog, QMessageBox
from PyQt5.QtGui import QDrag, QFont, QRegExpValidator

import json

import make_config_defs

import List
import ListItem

class PageFrame(QtWidgets.QFrame):
    def __init__(self, parent=None):
        super(PageFrame, self).__init__(parent)

        self.parent = parent

        self.setAcceptDrops(True)

        master_frame_layout = QtWidgets.QVBoxLayout()

        label_frame = QtWidgets.QFrame()
        label_frame_layout = QtWidgets.QHBoxLayout()

        page_name_label = QtWidgets.QLabel("Tab Name:")
        self.page_name_box = QtWidgets.QLineEdit()
        rx = QRegExp("[A-Za-z]+[A-Za-z0-9_]*")
        v = QRegExpValidator(rx, self)
        self.page_name_box.setValidator(v)
        self.page_name_box.editingFinished.connect(self.nameEdited)
        self.page_name_box.textChanged.connect(self.nameEdited)

        label_frame_layout.addWidget(page_name_label)
        label_frame_layout.addWidget(self.page_name_box)
        label_frame_layout.addStretch(1)
        label_frame.setLayout(label_frame_layout)

        page_frame = QtWidgets.QFrame()
        #page_frame.setStyleSheet("background-color:yellow;")
        page_layout = QtWidgets.QGridLayout()

        self.page_listboxes = []
        for i in range(12):
            name = "Box_%d" % (i)
            self.page_listboxes.append(List.List(name, parent=self))
            row = 0 if i < 6 else  1
            col = i if i < 6 else i - 6
            page_layout.addWidget(self.page_listboxes[i], row, col)    
        page_frame.setLayout(page_layout)

        #parent.statusbar.showMessage("FOO")

        bottom_frame = QtWidgets.QFrame()
        bottom_frame_layout = QtWidgets.QHBoxLayout()
        self.clear_button = QtWidgets.QPushButton("Clear")
        self.add_button = QtWidgets.QPushButton("Add")
        self.add_button.setEnabled(False)
        self.save_button = QtWidgets.QPushButton("Save")
        self.save_button.setEnabled(False)
        self.read_button = QtWidgets.QPushButton("Read")

        self.add_button.clicked.connect(self.add_pressed)
        self.clear_button.clicked.connect(self.clear)
        self.save_button.clicked.connect(self.save)
        self.read_button.clicked.connect(self.read)

        bottom_frame_layout.addWidget(self.clear_button)
        bottom_frame_layout.addWidget(self.add_button)
        bottom_frame_layout.addStretch(1)
        bottom_frame_layout.addWidget(self.read_button)
        bottom_frame_layout.addWidget(self.save_button)
        bottom_frame.setLayout(bottom_frame_layout)

        master_frame_layout.addWidget(label_frame)
        master_frame_layout.addWidget(page_frame)
        master_frame_layout.addWidget(bottom_frame)

        self.setLayout(master_frame_layout)


    def dropEvent(self, event):
#        print("dropEvent")
        super().dropEvent(event)
        event.accept()


    def clear(self):
        self.page_name_box.setText('')
        for box in self.page_listboxes:
            nitems = box.count()
            for _ in range(nitems):
                box.takeItem(0)


    def nameEdited(self):
        name = self.page_name_box.text()
        enable = True if name != '' else False
        self.save_button.setEnabled(enable)
        self.add_button.setEnabled(enable)
        pass


    def listboxes_empty(self):
        status = True
        for box in self.page_listboxes:
            status = status and (box.count() == 0)
        return status

    def name_set(self, name):
        if name == '':
            self.parent.statusbar.showMessage("Tab name is required")
            return False
        return True

    def add_pressed(self):
        name = self.page_name_box.text()
        if self.name_set(name):
            if self.listboxes_empty():
                self.parent.statusbar.showMessage("At least one parameter is required")
                return False
            ret = self.parent.add_page(name, self.page_listboxes)
            if ret:
                self.clear()
                return ret
        else:
            return False

    # but this is just 1 tab, not a page of tabs...
    def save(self):
        name = self.page_name_box.text()
        if self.name_set(name):
            if self.listboxes_empty():
                self.parent.statusbar.showMessage("At least one parameter is required")
            else:
                dict = {}
                groups = []
                n = 0
                for b in self.page_listboxes:
                    groups.append(self.parent.make_group(b, n))
                    n += 1
                dict['name'] = name
                dict['groups'] = groups

                options = QFileDialog.Options()
                options |= QFileDialog.DontUseNativeDialog
                suffixes = 'json file (*.json)'

                filename, _ = QFileDialog.getSaveFileName(self, "QFileDialog.getSaveFileName()", make_config_defs.page_tabs_dir, suffixes, options=options)
                if filename:
                    print(f"filename is {filename}")
                    if filename.endswith('.json'):
                        print(f"(has json) filename = {filename}")
                        pass
                    else:
                        filename += '.json'
                        print(f"(needs json) filename = {filename}")
                with open(filename, 'w') as outfile:
                    json.dump(dict, outfile)
        pass

    def read(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        suffixes = 'json file (*.json)'
        filename, _ = QFileDialog.getOpenFileName(self, "QFileDialog.getOpenFileName()", make_config_defs.page_tabs_dir, suffixes, options=options)
        if filename:
            self.clear()
            with open(filename) as f:
                data = json.load(f)
                self.page_name_box.setText(data['name'])
                groups = data['groups']
                box = 0
                for group in groups:
                    labels = group['labels']
                    for label in labels:
                        item = ListItem.ListItem(label['mib_name'], label['mib_id'], label['source'], label['unit'], 0)
                        self.page_listboxes[box].addItem(item)
                    box += 1
        pass
